# ################################################################# #
#### LOAD LIBRARY AND DEFINE CORE SETTINGS                       ####
# ################################################################# #

### Initialise
rm(list = ls())
library(apollo)

apollo_initialise()

### Set core controls
apollo_control = list(
  modelName  = "Hybrid_LC_with_OL",
  modelDescr = "Hybrid latent class choice model on drug choice data, using ordered measurement model for indicators",
  indivID    = "ID",
  mixing     = TRUE,
  nCores     = 5,
  outputDirectory = "output"
)

# ################################################################# #
#### LOAD DATA AND APPLY ANY TRANSFORMATIONS                     ####
# ################################################################# #

### Loading data from package
### if data is to be loaded from a file (e.g. called data.csv), 
### the code would be: database = read.csv("data.csv",header=TRUE)
database = apollo_drugChoiceData
### for data dictionary, use ?apollo_swissRouteChoiceData

# ################################################################# #
#### DEFINE MODEL PARAMETERS                                     ####
# ################################################################# #

### Vector of parameters, including any that are kept fixed in estimation
apollo_beta = c(bArt_A =.1, bArt_B = 0,
                bNov_A =.1, bNov_B = 0,
                bBes_A = 0, bBes_B =.1,
                bSup_A = 0, bSup_B =.1,
                bPai_A = 0, bPai_B =.1,
                bCH_A  = 0, bCH_B  = 0,
                bDK_A  = 0, bDK_B  = 0,
                bUS_A  = 0, bUS_B  = 0,
                bIN_A  = 0, bIN_B  = 0,
                bRU_A  = 0, bRU_B  = 0,
                bBR_A  = 0, bBR_B  = 0,
                bStd_A = 0, bStd_B = 0,
                bFas_A = 0, bFas_B = 0,
                bDou_A = 0, bDou_B = 0,
                bRsk_A = 0, bRsk_B = 0,
                bPri_A = 0, bPri_B = 0,
                lambda_A = 1, lambda_B = 1, 
                delta_A  = 0, delta_B  = 0, 
                zQuality    = 1, 
                zIngredient = 1, 
                zPatent     = 1, 
                zDominance  = 1, 
                sQuality    = 1, 
                sIngredient = 1, 
                sPatent     = 1, 
                sDominance  = 1)

### Vector with names (in quotes) of parameters to be kept fixed at their starting value in apollo_beta, use apollo_beta_fixed = c() if none
apollo_fixed = c("bArt_A", "bUS_A", "bStd_A", "bArt_B", "bUS_B", "bStd_B", "delta_A")

# ################################################################# #
#### DEFINE RANDOM COMPONENTS                                    ####
# ################################################################# #

### Set parameters for generating draws
apollo_draws = list(
  interDrawsType = "halton", 
  interNDraws    = 100,
  interNormDraws = c("eta")
)

### Create random parameters
apollo_randCoeff=function(apollo_beta, apollo_inputs){
  randcoeff = list()
  randcoeff[["LV"]] = eta
  return(randcoeff)
}

# ################################################################# #
#### DEFINE LATENT CLASS COMPONENTS                              ####
# ################################################################# #

apollo_lcPars=function(apollo_beta, apollo_inputs){
  lcpars = list()
  
  lcpars[["bArt"]] = list(bArt_A, bArt_B)
  lcpars[["bNov"]] = list(bNov_A, bNov_B)
  lcpars[["bBes"]] = list(bBes_A, bBes_B)
  lcpars[["bSup"]] = list(bSup_A, bSup_B)
  lcpars[["bPai"]] = list(bPai_A, bPai_B)
  lcpars[["bCH" ]] = list(bCH_A , bCH_B )
  lcpars[["bDK" ]] = list(bDK_A , bDK_B )
  lcpars[["bUS" ]] = list(bUS_A , bUS_B )
  lcpars[["bIN" ]] = list(bIN_A , bIN_B )
  lcpars[["bRU" ]] = list(bRU_A , bRU_B )
  lcpars[["bBR" ]] = list(bBR_A , bBR_B )
  lcpars[["bStd"]] = list(bStd_A, bStd_B)
  lcpars[["bFas"]] = list(bFas_A, bFas_B)
  lcpars[["bDou"]] = list(bDou_A, bDou_B)
  lcpars[["bRsk"]] = list(bRsk_A, bRsk_B)
  lcpars[["bPri"]] = list(bPri_A, bPri_B)
  lcpars[["lambda"]] = list(lambda_A, lambda_B)
  
  ### Settings for class allocation models
  classAlloc_settings = list(
    classes      = c(class_a=1, class_b=2), 
    V            = V  
  )
  
  lcpars[["pi_values"]] = apollo_classAlloc(classAlloc_settings)
  
  return(lcpars)
}

# ################################################################# #
#### GROUP AND VALIDATE INPUTS                                   ####
# ################################################################# #

apollo_inputs = apollo_validateInputs()

# ################################################################# #
#### DEFINE MODEL AND LIKELIHOOD FUNCTION                        ####
# ################################################################# #

apollo_probabilities=function(apollo_beta, apollo_inputs, functionality="estimate"){
  
  ### Initialise
  apollo_attach(apollo_beta, apollo_inputs)
  on.exit(apollo_detach(apollo_beta, apollo_inputs))
  P = list()
  
  ### Likelihood of indicators
  normalDensity_settings1 = list(outcomeNormal = attitude_quality, 
                                 xNormal       = zQuality*LV, 
                                 mu            = 0, 
                                 sigma         = sQuality, 
                                 rows          = (task==1))
  normalDensity_settings2 = list(outcomeNormal = attitude_ingredients, 
                                 xNormal       = zIngredient*LV, 
                                 mu            = 0, 
                                 sigma         = sIngredient, 
                                 rows          = (task==1))
  normalDensity_settings3 = list(outcomeNormal = attitude_patent, 
                                 xNormal       = zPatent*LV, 
                                 mu            = 0, 
                                 sigma         = sPatent, 
                                 rows          = (task==1))
  normalDensity_settings4 = list(outcomeNormal = attitude_dominance, 
                                 xNormal       = zDominance*LV, 
                                 mu            = 0, 
                                 sigma         = sDominance, 
                                 rows          = (task==1))
  P[["indic_quality"]]     = apollo_normalDensity(normalDensity_settings1, functionality)
  P[["indic_ingredients"]] = apollo_normalDensity(normalDensity_settings2, functionality)
  P[["indic_patent"]]      = apollo_normalDensity(normalDensity_settings3, functionality)
  P[["indic_dominance"]]   = apollo_normalDensity(normalDensity_settings4, functionality)
  P[["indic_quality"]]     = apollo_panelProd(P[["indic_quality"]]    , apollo_inputs, functionality)
  P[["indic_ingredients"]] = apollo_panelProd(P[["indic_ingredients"]], apollo_inputs, functionality)
  P[["indic_patent"]]      = apollo_panelProd(P[["indic_patent"]]     , apollo_inputs, functionality)
  P[["indic_dominance"]]   = apollo_panelProd(P[["indic_dominance"]]  , apollo_inputs, functionality)
  
  ### Likelihood of choices inside each class
  S <- 2
  for(s in 1:S){
    ### Utilities for alternatives
    V = list()
    V[["alt1"]] = bArt[[s]]*(brand_1=="Artemis")   + bNov[[s]]*(brand_1=="Novum")                                         + bCH[[s]]*(country_1=="Switzerland") + bDK[[s]]*(country_1=="Denmark") + bUS[[s]]*(country_1=="USA")                                     + bStd[[s]]*(char_1=="standard") + bFas[[s]]*(char_1=="fast acting") + bDou[[s]]*(char_1=="double strength") + bRsk[[s]]*side_effects_1 + bPri[[s]]*price_1 + lambda[[s]]*LV
    V[["alt2"]] = bArt[[s]]*(brand_2=="Artemis")   + bNov[[s]]*(brand_2=="Novum")                                         + bCH[[s]]*(country_2=="Switzerland") + bDK[[s]]*(country_2=="Denmark") + bUS[[s]]*(country_2=="USA")                                     + bStd[[s]]*(char_2=="standard") + bFas[[s]]*(char_2=="fast acting") + bDou[[s]]*(char_2=="double strength") + bRsk[[s]]*side_effects_2 + bPri[[s]]*price_2 + lambda[[s]]*LV
    V[["alt3"]] = bBes[[s]]*(brand_3=="BestValue") + bSup[[s]]*(brand_3=="Supermarket") + bPai[[s]]*(brand_3=="PainAway") + bUS[[s]]*(country_3=="USA")         + bIN[[s]]*(country_3=="India")   + bRU[[s]]*(country_3=="Russia") + bBR[[s]]*(country_3=="Brazil") + bStd[[s]]*(char_3=="standard") + bFas[[s]]*(char_3=="fast acting")                                         + bRsk[[s]]*side_effects_3 + bPri[[s]]*price_3
    V[["alt4"]] = bBes[[s]]*(brand_4=="BestValue") + bSup[[s]]*(brand_4=="Supermarket") + bPai[[s]]*(brand_4=="PainAway") + bUS[[s]]*(country_4=="USA")         + bIN[[s]]*(country_4=="India")   + bRU[[s]]*(country_4=="Russia") + bBR[[s]]*(country_4=="Brazil") + bStd[[s]]*(char_4=="standard") + bFas[[s]]*(char_4=="fast acting")                                         + bRsk[[s]]*side_effects_4 + bPri[[s]]*price_4
    ### Define settings for MNL model component
    mnl_settings = list(
      alternatives = c(alt1=1, alt2=2, alt3=3, alt4=4),
      choiceVar    = best,
      utilities    = V
    )
    ### Compute within-class choice probabilities using MNL model
    P[[paste0("Class_",s)]] = apollo_mnl(mnl_settings, functionality)
    
    ### Take product across observation for same individual
    P[[paste0("Class_",s)]] = apollo_panelProd(P[[paste0("Class_",s)]], apollo_inputs ,functionality)
  }
  
  ### Compute latent class model probabilities
  lc_settings  = list(inClassProb = P[paste0("Class_", 1:S)], classProb=pi_values)
  P[["choice"]] = apollo_lc(lc_settings, apollo_inputs, functionality)
  
  ### Comment out as necessary
  P = apollo_combineModels(P, apollo_inputs, functionality)
  P = apollo_avgInterDraws(P, apollo_inputs, functionality)
  P = apollo_prepareProb(P, apollo_inputs, functionality)
  return(P)
}

# ################################################################# #
#### CALCULATE LL AT STARTING VALUES                             ####
# ################################################################# #

apollo_llCalc(apollo_beta, apollo_probabilities, apollo_inputs)

# ################################################################# #
#### MODEL ESTIMATION                                            ####
# ################################################################# #

model = apollo_estimate(apollo_beta, apollo_fixed, apollo_probabilities, apollo_inputs)

# ################################################################# #
#### MODEL OUTPUTS                                               ####
# ################################################################# #

# ----------------------------------------------------------------- #
#---- FORMATTED OUTPUT (TO SCREEN)                               ----
# ----------------------------------------------------------------- #

apollo_modelOutput(model)

# ----------------------------------------------------------------- #
#---- FORMATTED OUTPUT (TO FILE, using model name)               ----
# ----------------------------------------------------------------- #

apollo_saveOutput(model)

